# -*- coding: utf-8 -*-

"""
    dropbox_test_api

    This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

from dropbox_test_api.api_helper import APIHelper
from dropbox_test_api.configuration import Configuration
from dropbox_test_api.controllers.base_controller import BaseController
from dropbox_test_api.models.o_auth_token import OAuthToken
from dropbox_test_api.exceptions.o_auth_provider_exception import OAuthProviderException

class OAuthAuthorizationController(BaseController):

    """A Controller to access Endpoints in the dropbox_test_api API."""


    def request_token(self,
                      authorization,
                      code,
                      redirect_uri,
                      _optional_form_parameters=None):
        """Does a POST request to /token.

        Create a new OAuth 2 token.

        Args:
            authorization (string): Authorization header in Basic auth format
            code (string): Authorization Code
            redirect_uri (string): Redirect Uri
            _optional_form_parameters (Array, optional): Additional optional
                form parameters are supported by this endpoint

        Returns:
            OAuthToken: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _url_path = '/token'
        _query_builder = Configuration.get_base_uri(Configuration.Server.TOKEN_AUTH)
        _query_builder += _url_path
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json',
            'Authorization': authorization
        }

        # Prepare form parameters
        _form_parameters = {
            'grant_type': 'authorization_code',
            'code': code,
            'redirect_uri': redirect_uri
        }
        if _form_parameters != None and _optional_form_parameters != None:
            _form_parameters.update(APIHelper.form_encode_parameters(_optional_form_parameters,
                Configuration.array_serialization))
        _form_parameters = APIHelper.form_encode_parameters(_form_parameters,
            Configuration.array_serialization)

        # Prepare and execute request
        _request = self.http_client.post(_query_url, headers=_headers, parameters=_form_parameters)
        _context = self.execute_request(_request)

        # Endpoint and global error handling using HTTP status codes.
        if _context.response.status_code == 400:
            raise OAuthProviderException('OAuth 2 provider returned an error.', _context)
        elif _context.response.status_code == 401:
            raise OAuthProviderException('OAuth 2 provider says client authentication failed.', _context)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, OAuthToken.from_dictionary)

    def refresh_token(self,
                      authorization,
                      refresh_token,
                      scope=None,
                      _optional_form_parameters=None):
        """Does a POST request to /token.

        Obtain a new access token using a refresh token

        Args:
            authorization (string): Authorization header in Basic auth format
            refresh_token (string): Refresh token
            scope (string, optional): Requested scopes as a space-delimited
                list.
            _optional_form_parameters (Array, optional): Additional optional
                form parameters are supported by this endpoint

        Returns:
            OAuthToken: Response from the API. 

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _url_path = '/token'
        _query_builder = Configuration.get_base_uri(Configuration.Server.TOKEN_AUTH)
        _query_builder += _url_path
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json',
            'Authorization': authorization
        }

        # Prepare form parameters
        _form_parameters = {
            'grant_type': 'refresh_token',
            'refresh_token': refresh_token,
            'scope': scope
        }
        if _form_parameters != None and _optional_form_parameters != None:
            _form_parameters.update(APIHelper.form_encode_parameters(_optional_form_parameters,
                Configuration.array_serialization))
        _form_parameters = APIHelper.form_encode_parameters(_form_parameters,
            Configuration.array_serialization)

        # Prepare and execute request
        _request = self.http_client.post(_query_url, headers=_headers, parameters=_form_parameters)
        _context = self.execute_request(_request)

        # Endpoint and global error handling using HTTP status codes.
        if _context.response.status_code == 400:
            raise OAuthProviderException('OAuth 2 provider returned an error.', _context)
        elif _context.response.status_code == 401:
            raise OAuthProviderException('OAuth 2 provider says client authentication failed.', _context)
        self.validate_response(_context)

        # Return appropriate type
        return APIHelper.json_deserialize(_context.response.raw_body, OAuthToken.from_dictionary)
